#version 400

uniform samplerCube cubeTexSampler;
uniform float matReflectFactor; // The direction of the reflected ray
uniform vec4 matColor; ///< Color of the object's "Tint"
uniform vec3 cameraPosWS;

// varyings
in vec3 varPosWS;
in vec3 varNormalWS;

// output-color of fragment
layout(location = 0) out vec4 fragColor;

void main()
{

	vec3 viewDirWS = normalize(varPosWS - cameraPosWS);
	
	//vec3 refractDir = refract(viewDirWS, -varNormalWS, 0.5);
	vec3 refractDir = refract(viewDirWS, varNormalWS, 0.5);
	vec3 reflectDir = reflect(viewDirWS, varNormalWS);

	vec4 refractColor = texture(cubeTexSampler, refractDir);
	vec4 reflectColor = texture(cubeTexSampler, reflectDir);

	float fresnel = abs(dot(viewDirWS, varNormalWS));
    fresnel *= 0.8;
    fresnel = clamp(fresnel, 0.1, 0.9);
	//TODO do not MIX ALPHA CHANNEL
	vec4 tmp = mix(refractColor, reflectColor, fresnel);
	fragColor = mix(matColor, tmp, matReflectFactor);
}