#version 400

uniform mat4 pvmtMatrix;
uniform mat4 vmtMatrix;
uniform mat4 vmtInvTranspMatrix;
uniform mat4 boneMatrices[60];

uniform float near;
uniform float far;

layout (location = 0) in vec4 vertexPosition; 
layout (location = 2) in vec4 vertexNormal;
layout (location = 6) in vec4 vertexBoneWeights;
layout (location = 7) in ivec4 vertexBoneIndices; //NOTE: although the indices were uploaded as GL_FLOAT, they are integers

out vec3 varNormal;   ///< vertex normal in viewing-space
out float varDepth;

void main()
{	
	vec4 boneWeights = vertexBoneWeights;
	boneWeights.w = 1.0 - dot(boneWeights.xyz, vec3(1.0, 1.0, 1.0));

	vec4 newPosition = boneWeights.x * (boneMatrices[vertexBoneIndices.x] * vertexPosition);
	newPosition += boneWeights.y * (boneMatrices[vertexBoneIndices.y] * vertexPosition);
	newPosition += boneWeights.z * (boneMatrices[vertexBoneIndices.z] * vertexPosition);
	newPosition += boneWeights.w * (boneMatrices[vertexBoneIndices.w] * vertexPosition);

	vec3 newNormal = boneWeights.x * (mat3(boneMatrices[vertexBoneIndices.x]) * vertexNormal.xyz);
	newNormal += boneWeights.y * (mat3(boneMatrices[vertexBoneIndices.y]) * vertexNormal.xyz);
	newNormal += boneWeights.z * (mat3(boneMatrices[vertexBoneIndices.z]) * vertexNormal.xyz);
	newNormal += boneWeights.w * (mat3(boneMatrices[vertexBoneIndices.w]) * vertexNormal.xyz);


	varNormal = normalize(mat3(vmtInvTranspMatrix) * newNormal);
	varNormal = varNormal * 0.5 + 0.5;
	
	vec4 viewSpacePos = vmtMatrix * newPosition;
	
    varDepth = (-viewSpacePos.z-near)/(far-near); // will map near..far to 0..1

	gl_Position = pvmtMatrix * newPosition;
}
