#version 400

uniform mat4 projectionMatrix;
uniform mat4 pvmtMatrix;
uniform mat4 modelMatrix;
uniform mat4 vmtMatrix;
uniform mat3 normalMatrix;
uniform mat4 boneMatrices[60];
uniform mat4 textureModelMatrix; // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)
//uniform mat4 dirLightTextureModelMatrix;  // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)
uniform mat4 spotLightTextureModelMatrix0; // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)
uniform mat4 spotLightTextureModelMatrix1; // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)

layout (location = 0) in vec4 vertexPosition; 
layout (location = 1) in vec2 vertexTexCoord;
layout (location = 2) in vec3 vertexNormal;
layout (location = 6) in vec4 vertexBoneWeights;
layout (location = 7) in ivec4 vertexBoneIndices; //NOTE: although the indices were uploaded as GL_FLOAT, they are integers

out vec2 varTexCoords;
//out vec4 varDirLightShadowMapTexCoords;
out vec4 varSpotLightShadowMapTexCoords0;
out vec4 varSpotLightShadowMapTexCoords1;
out vec3 varPositionVS; ///< vertex position in viewing-space
out vec3 varNormalVS;   ///< vertex normal in viewing-space

void main()
{	
	vec4 boneWeights = vertexBoneWeights;
	boneWeights.w = 1.0 - dot(boneWeights.xyz, vec3(1.0, 1.0, 1.0));

	vec4 newPosition = boneWeights.x * (boneMatrices[vertexBoneIndices.x] * vertexPosition);
	newPosition += boneWeights.y * (boneMatrices[vertexBoneIndices.y] * vertexPosition);
	newPosition += boneWeights.z * (boneMatrices[vertexBoneIndices.z] * vertexPosition);
	newPosition += boneWeights.w * (boneMatrices[vertexBoneIndices.w] * vertexPosition);

	vec3 newNormal = boneWeights.x * (mat3(boneMatrices[vertexBoneIndices.x]) * vertexNormal);
	newNormal += boneWeights.y * (mat3(boneMatrices[vertexBoneIndices.y]) * vertexNormal);
	newNormal += boneWeights.z * (mat3(boneMatrices[vertexBoneIndices.z]) * vertexNormal);
	newNormal += boneWeights.w * (mat3(boneMatrices[vertexBoneIndices.w]) * vertexNormal);

	varTexCoords = vertexTexCoord;
	varNormalVS = normalMatrix * newNormal;
	varPositionVS = (vmtMatrix * newPosition).xyz;

	//varDirLightShadowMapTexCoords = dirLightTextureModelMatrix * newPosition;
	varSpotLightShadowMapTexCoords0 = spotLightTextureModelMatrix0 * newPosition;
	varSpotLightShadowMapTexCoords1 = spotLightTextureModelMatrix1 * newPosition;

	gl_Position = pvmtMatrix * newPosition;
}


//"	 vec4 normal = vec4(inNormal, 0.0);\n"
//"\n"
//"  vec4 newNormal = boneWeights.x * boneMatrices[inBoneIndices.x] * normal;\n"
//"  newNormal += boneWeights.y * boneMatrices[inBoneIndices.y] * normal;\n"
//"  newNormal += boneWeights.z * boneMatrices[inBoneIndices.z] * normal;\n"
//"  newNormal += boneWeights.w * boneMatrices[inBoneIndices.w] * normal;\n"
//"	 worldNormal = (modelMatrix * newNormal).xyz;\n"
//"\n"