#version 400

uniform mat4 pvmtMatrix;
uniform mat4 mtMatrix;
uniform mat3 mtNormalMatrix;
uniform mat4 textureModelMatrix; // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)
//uniform mat4 dirLightTextureModelMatrix;  // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)
uniform mat4 spotLightTextureModelMatrix0; // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)
uniform mat4 spotLightTextureModelMatrix1; // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)

layout (location = 0) in vec4 vertexPosition; 
layout (location = 1) in vec2 vertexTexCoord;
layout (location = 2) in vec3 vertexNormal;
layout (location = 8) in vec3 vertexTangent;
layout (location = 9) in vec3 vertexBitangent;

out vec2 varTexCoords;
//out vec4 varDirLightShadowMapTexCoords;
out vec4 varSpotLightShadowMapTexCoords0;
out vec4 varSpotLightShadowMapTexCoords1;

out vec3 varNormal;
out vec3 varTangent;
out vec3 varVertexPosWS;

void main()
{	
	// calculate transformation MATRICES to and from TANGENT SPACE
	
	// The TBN matrix is an orthogonal matrix (because the T, B, and N vectors
	// define a coordinate system). The transpose of an orthogonal matrix is
	// it's inverse matrix. 
	// Therefore transpose(OStoTS) == inverse(OStoTS)

	vec3 normalOS		= vertexNormal;
	vec3 tangentOS		= vertexTangent;
	//vec3 bitangentOS	= vertexBitangent;
	vec3 bitangentOS	= cross(normalOS, tangentOS);
	mat3 matrixTStoOS	= mat3(tangentOS, bitangentOS, normalOS);
	mat3 matrixOStoTS	= transpose(matrixTStoOS);

	vec3 normalWS		= normalize(mtNormalMatrix * normalOS);
	vec3 tangentWS		= normalize(mtNormalMatrix * tangentOS);
	//vec3 bitangentWS	= normalize(mtNormalMatrix * bitangentOS);
	vec3 bitangentWS	= cross(normalWS, tangentWS);
	//mat3 matrixTStoWS	= mat3(tangentWS, bitangentWS, normalWS);
	//mat3 matrixWStoTS	= transpose(matrixTStoWS);

	varTangent = tangentWS;
	varNormal = normalWS;

	// calculate light direction and eye vector in WORLD SPACE, transform light direction L and eye vector V to TANGENT SPACE and normalize
	varVertexPosWS = (mtMatrix * vertexPosition).xyz;


	// send the rest down the pipeline
	varTexCoords = vertexTexCoord;
	//varDirLightShadowMapTexCoords = dirLightTextureModelMatrix * vertexPosition;
	varSpotLightShadowMapTexCoords0 = spotLightTextureModelMatrix0 * vertexPosition;
	varSpotLightShadowMapTexCoords1 = spotLightTextureModelMatrix1 * vertexPosition;
	gl_Position = pvmtMatrix * vertexPosition;
}
