#version 400

uniform mat4 pvmtMatrix;
uniform mat4 mtMatrix;
uniform mat3 mtNormalMatrix;
uniform mat4 boneMatrices[60];
uniform mat4 textureModelMatrix; // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)
uniform mat4 dirLightTextureModelMatrix;  // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)
uniform mat4 spotLightTextureModelMatrix0; // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)
uniform mat4 spotLightTextureModelMatrix1; // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)

uniform vec3 eyePosWS;

// up to two directional light
const unsigned int DIRLIGHTS = 1;
uniform unsigned int numDirLights;
uniform vec3 dirLightDirection[DIRLIGHTS];		///< light-direction in WORLD, must be normalized

// up to four positional lights; we need diffuse color and specular color in fragment shader only
const unsigned int POSLIGHTS = 6;
uniform unsigned int numPosLights;
uniform vec3 posLightPosition[POSLIGHTS]; //< positional light positions in WORLD SPACE
uniform float posLightLinearAttenuation[POSLIGHTS];
uniform float posLightQuadraticAttenuation[POSLIGHTS];

// up to 2 spotlights
const unsigned int SPOTLIGHTS = 1;
uniform unsigned int numSpotLights;
uniform vec3 spotLightPosition[SPOTLIGHTS];		///< spotlight's position in WORLD SPACE
uniform float spotLightLinearAttenuation[SPOTLIGHTS];
uniform float spotLightQuadraticAttenuation[SPOTLIGHTS];
uniform vec3 spotLightDirection[SPOTLIGHTS];	///< spotlight's direction in WORLD SPACE, normalized

layout (location = 0) in vec4 vertexPosition; 
layout (location = 1) in vec2 vertexTexCoord;
layout (location = 2) in vec3 vertexNormal;
layout (location = 6) in vec4 vertexBoneWeights;
layout (location = 7) in ivec4 vertexBoneIndices; //NOTE: although the indices were uploaded as GL_FLOAT, they are integers
layout (location = 8) in vec3 vertexTangent;
layout (location = 9) in vec3 vertexBitangent;

out vec2 varTexCoords;
//out vec4 varDirLightShadowMapTexCoords;
out vec4 varSpotLightShadowMapTexCoords0;
out vec4 varSpotLightShadowMapTexCoords1;
//out vec3 varDirLightL[DIRLIGHTS];			///< to light direction in tangent space
//out vec3 varDirLightH[DIRLIGHTS];			///< half vector in tangent space
out vec3 varPosLightL[POSLIGHTS];			///< to light direction in tangent space
out vec3 varPosLightH[POSLIGHTS];			///< half vector in tangent space
out float varPosLightAtt[POSLIGHTS];		///< combined attenuation 
out vec3 varSpotLightL[SPOTLIGHTS];			///< to light direction in tangent space
out vec3 varSpotLightH[SPOTLIGHTS];			///< half vector in tangent space
out float varSpotLightAtt[SPOTLIGHTS];		///< combined attenuation 
out vec3 varSpotLightDir[SPOTLIGHTS];		///< spot light's light-direction in tangent space

void main()
{	
	// do animation: vertexPosition => newPosition, vertexNormal => newNormal

	vec4 boneWeights = vertexBoneWeights;
	boneWeights.w = 1.0 - dot(boneWeights.xyz, vec3(1.0, 1.0, 1.0));

	vec4 newPosition = boneWeights.x * (boneMatrices[vertexBoneIndices.x] * vertexPosition);
	newPosition += boneWeights.y * (boneMatrices[vertexBoneIndices.y] * vertexPosition);
	newPosition += boneWeights.z * (boneMatrices[vertexBoneIndices.z] * vertexPosition);
	newPosition += boneWeights.w * (boneMatrices[vertexBoneIndices.w] * vertexPosition);

	vec3 newNormal = boneWeights.x * (mat3(boneMatrices[vertexBoneIndices.x]) * vertexNormal);
	newNormal += boneWeights.y * (mat3(boneMatrices[vertexBoneIndices.y]) * vertexNormal);
	newNormal += boneWeights.z * (mat3(boneMatrices[vertexBoneIndices.z]) * vertexNormal);
	newNormal += boneWeights.w * (mat3(boneMatrices[vertexBoneIndices.w]) * vertexNormal);


	// calculate transformation MATRICES to and from TANGENT SPACE
	
	// The TBN matrix is an orthogonal matrix (because the T, B, and N vectors
	// define a coordinate system). The transpose of an orthogonal matrix is
	// it's inverse matrix. 
	// Therefore transpose(OStoTS) == inverse(OStoTS)

	vec3 normalOS		= newNormal;
	vec3 tangentOS		= vertexTangent;
	vec3 bitangentOS	= vertexBitangent;
	mat3 matrixTStoOS	= mat3(tangentOS, bitangentOS, normalOS);
	mat3 matrixOStoTS	= transpose(matrixTStoOS);

	vec3 normalWS		= normalize(mtNormalMatrix * normalOS);
	vec3 tangentWS		= normalize(mtNormalMatrix * tangentOS);
	vec3 bitangentWS	= normalize(mtNormalMatrix * bitangentOS);
	mat3 matrixTStoWS	= mat3(tangentWS, bitangentWS, normalWS);
	mat3 matrixWStoTS	= transpose(matrixTStoWS);


	// calculate light direction and eye vector in WORLD SPACE, transform light direction L and eye vector V to TANGENT SPACE and normalize
	vec3 vertexPosWS = (mtMatrix * newPosition).xyz;
	vec3 toEyeDirWS = normalize(eyePosWS - vertexPosWS);
	vec3 toEyeDirTS = normalize(matrixWStoTS * toEyeDirWS);


	vec3 toLightWS, toLightDirNrmWS, toLightDirNrmTS, halfVectorTS;
	float distSq, dist;

	//// directional lights contribution
	//for (int i=0; i < numDirLights; ++i)
	//{
	//	toLightDirNrmWS = -dirLightDirection[i];
	//	toLightDirNrmTS = normalize(matrixWStoTS * toLightDirNrmWS);
	//	halfVectorTS = normalize(toEyeDirTS + toLightDirNrmTS);

	//	varDirLightL[i] = toLightDirNrmTS;
	//	varDirLightH[i] = halfVectorTS;
	//}

	// point lights contribution
	for (int i=0; i < numPosLights; ++i)
	{
		toLightWS = posLightPosition[i] - vertexPosWS;
		distSq = dot(toLightWS, toLightWS);
		dist = sqrt(distSq);
		toLightDirNrmWS = normalize(toLightWS);
		toLightDirNrmTS = normalize(matrixWStoTS * toLightDirNrmWS);
		halfVectorTS = normalize(toEyeDirTS + toLightDirNrmTS);

		varPosLightL[i] = toLightDirNrmTS;
		varPosLightH[i] = halfVectorTS;
		varPosLightAtt[i] = 1.0 + posLightLinearAttenuation[i] * dist + posLightQuadraticAttenuation[i] * distSq;
	}

	// spot lights contribution
	for (int i=0; i < numSpotLights; ++i)
	{
		toLightWS = spotLightPosition[i] - vertexPosWS;
		distSq = dot(toLightWS, toLightWS);
		dist = sqrt(distSq);
		toLightDirNrmWS = normalize(toLightWS);
		toLightDirNrmTS = normalize(matrixWStoTS * toLightDirNrmWS);
		halfVectorTS = normalize(toEyeDirTS + toLightDirNrmTS);

		varSpotLightL[i] = toLightDirNrmTS;
		varSpotLightH[i] = halfVectorTS;
		varSpotLightAtt[i] = 1.0 + spotLightLinearAttenuation[i] * dist + spotLightQuadraticAttenuation[i] * distSq;
		varSpotLightDir[i] = normalize(matrixWStoTS * spotLightDirection[i]);
	}

	// send the rest down the pipeline
	varTexCoords = vertexTexCoord;
	//varDirLightShadowMapTexCoords = dirLightTextureModelMatrix * newPosition;
	varSpotLightShadowMapTexCoords0 = spotLightTextureModelMatrix0 * newPosition;
	varSpotLightShadowMapTexCoords1 = spotLightTextureModelMatrix1 * newPosition;
	gl_Position = pvmtMatrix * newPosition;
}
