#version 400 core

uniform mat4 viewMatrix;
uniform mat4 projMatrix;

layout(points) in;
layout(line_strip, max_vertices=20) out;

in vec3 varFront[];
in vec3 varUp[];
in vec3 varSide[];

out vec4 varVertexColor;

void main(void) 
{
	const float arrowLength = 1.0;
	const float arrowHeadLength = 0.05;

	mat4 pvM = projMatrix * viewMatrix;

	vec4 frontVec = vec4(varFront[0], 0);
	vec4 upVec = vec4(varUp[0], 0);
	vec4 sideVec = vec4(varSide[0], 0);

	// FRONT VECTOR (blue)

	gl_Position = pvM * gl_in[0].gl_Position;
	varVertexColor = vec4(0.0, 0.0, 1.0, 1.0);
	EmitVertex();
	
	gl_Position = pvM * (gl_in[0].gl_Position + arrowLength * frontVec);
	varVertexColor = vec4(0.0, 0.0, 1.0, 1.0);
	EmitVertex();
	
	gl_Position = pvM * (gl_in[0].gl_Position + arrowLength * frontVec + arrowHeadLength * sideVec - arrowHeadLength * upVec - arrowHeadLength * frontVec);
	varVertexColor = vec4(0.0, 0.0, 1.0, 1.0);
	EmitVertex();
	
	gl_Position = pvM * (gl_in[0].gl_Position + arrowLength * frontVec);
	varVertexColor = vec4(0.0, 0.0, 1.0, 1.0);
	EmitVertex();
	
	gl_Position = pvM * (gl_in[0].gl_Position + arrowLength * frontVec - arrowHeadLength * sideVec + arrowHeadLength * upVec - arrowHeadLength * frontVec);
	varVertexColor = vec4(0.0, 0.0, 1.0, 1.0);
	EmitVertex();
	
	gl_Position = pvM * (gl_in[0].gl_Position + arrowLength * frontVec);
	varVertexColor = vec4(0.0, 0.0, 1.0, 1.0);
	EmitVertex();
	
	gl_Position = pvM * gl_in[0].gl_Position;
	varVertexColor = vec4(0.0, 0.0, 1.0, 1.0);
	EmitVertex();

	// UP VECTOR (green)	

	gl_Position = pvM * gl_in[0].gl_Position;
	varVertexColor = vec4(0.0, 1.0, 0.0, 1.0);
	EmitVertex();

	gl_Position = pvM * (gl_in[0].gl_Position + arrowLength * upVec);
	varVertexColor = vec4(0.0, 1.0, 0.0, 1.0);
	EmitVertex();
	
	gl_Position = pvM * (gl_in[0].gl_Position + arrowLength * upVec + arrowHeadLength * sideVec - arrowHeadLength * frontVec - arrowHeadLength * upVec);
	varVertexColor = vec4(0.0, 1.0, 0.0, 1.0);
	EmitVertex();
	
	gl_Position = pvM * (gl_in[0].gl_Position + arrowLength * upVec);
	varVertexColor = vec4(0.0, 1.0, 0.0, 1.0);
	EmitVertex();
	
	gl_Position = pvM * (gl_in[0].gl_Position + arrowLength * upVec - arrowHeadLength * sideVec + arrowHeadLength * frontVec - arrowHeadLength * upVec);
	varVertexColor = vec4(0.0, 1.0, 0.0, 1.0);
	EmitVertex();
	
	gl_Position = pvM * (gl_in[0].gl_Position + arrowLength * upVec);
	varVertexColor = vec4(0.0, 1.0, 0.0, 1.0);
	EmitVertex();

	gl_Position = pvM * gl_in[0].gl_Position;
	varVertexColor = vec4(0.0, 1.0, 0.0, 1.0);
	EmitVertex();
	
	// SIDE VECTOR (red)	

	gl_Position = pvM * gl_in[0].gl_Position;
	varVertexColor = vec4(1.0, 0.0, 0.0, 1.0);
	EmitVertex();
	
	// #14
	gl_Position = pvM * (gl_in[0].gl_Position + arrowLength * sideVec);
	varVertexColor = vec4(1.0, 0.0, 0.0, 1.0);
	EmitVertex();
	
	// #15
	gl_Position = pvM * (gl_in[0].gl_Position + arrowLength * sideVec + arrowHeadLength * upVec - arrowHeadLength * frontVec - arrowHeadLength * sideVec);
	varVertexColor = vec4(1.0, 0.0, 0.0, 1.0);
	EmitVertex();
	
	// #16
	gl_Position = pvM * (gl_in[0].gl_Position + arrowLength * sideVec);
	varVertexColor = vec4(1.0, 0.0, 0.0, 1.0);
	EmitVertex();
	
	// #17
	gl_Position = pvM * (gl_in[0].gl_Position + arrowLength * sideVec - arrowHeadLength * upVec + arrowHeadLength * frontVec - arrowHeadLength * sideVec);
	varVertexColor = vec4(1.0, 0.0, 0.0, 1.0);
	EmitVertex();
	
	// #18
	gl_Position = pvM * (gl_in[0].gl_Position + arrowLength * sideVec);
	varVertexColor = vec4(1.0, 0.0, 0.0, 1.0);
	EmitVertex();
	
	
	EndPrimitive();
}