#version 330 core

uniform mat4 mtMatrix; //model matrix (passed per object)
uniform mat4 boneMatrices[60];

out vec2 geoTexCoords;

layout (location = 0) in vec4 vertexPosition;
layout (location = 1) in vec2 vertexTexCoord;
layout (location = 2) in vec3 vertexNormal;
layout (location = 6) in vec4 vertexBoneWeights;
layout (location = 7) in ivec4 vertexBoneIndices; //NOTE: although the indices were uploaded as GL_FLOAT, they are integers

void main(void) 
{
	vec4 boneWeights = vertexBoneWeights;
	boneWeights.w = 1.0 - dot(boneWeights.xyz, vec3(1.0, 1.0, 1.0));

	vec4 newPosition = boneWeights.x * (boneMatrices[vertexBoneIndices.x] * vertexPosition);
	newPosition += boneWeights.y * (boneMatrices[vertexBoneIndices.y] * vertexPosition);
	newPosition += boneWeights.z * (boneMatrices[vertexBoneIndices.z] * vertexPosition);
	newPosition += boneWeights.w * (boneMatrices[vertexBoneIndices.w] * vertexPosition);

	geoTexCoords = vertexTexCoord;

	// transform vertex to world space
	gl_Position = mtMatrix * newPosition;

}