#version 330 core
layout(triangles) in;
//3 vertices per tri, output 6 tris (1 for each cm-face)
layout(triangle_strip, max_vertices=18) out;

// projection*view matrices for each cube face
// contains P*V[i], transforms from WS to cubemap-face i
uniform mat4 cmMat[6];


uniform vec2 nearFar; // near and far plane for cm-cams
uniform vec4 lightPos; // world space light position

in vec2 geoTexCoords[];
in vec4 geoDirLightShadowMapTexCoords[];
in vec4 geoSpotLightShadowMapTexCoords0[];
in vec4 geoSpotLightShadowMapTexCoords1[];
in vec3 geoPositionVS[]; ///< vertex position in viewing-space
in vec3 geoNormalVS[];   ///< vertex normal in viewing-space


out vec4 WS_pos_from_GS;
out vec2 nF;
out vec4 lP;
out vec2 fragTexCoords;
out vec4 varDirLightShadowMapTexCoords;
out vec4 varSpotLightShadowMapTexCoords0;
out vec4 varSpotLightShadowMapTexCoords1;
out vec3 varPositionVS; ///< vertex position in viewing-space
out vec3 varNormalVS;   ///< vertex normal in viewing-space



void main(void) 
{
	nF = nearFar;
	lP = lightPos;

	
  //iterate over the 6 cubemap faces
	for(gl_Layer=0; gl_Layer<6; ++gl_Layer) 
	{
		for(int tri_vert=0; tri_vert<3; ++tri_vert) 
		{
			// passthrough tex coordinates
			fragTexCoords = geoTexCoords[tri_vert];

			varDirLightShadowMapTexCoords = geoDirLightShadowMapTexCoords[tri_vert];
			varSpotLightShadowMapTexCoords0 = geoSpotLightShadowMapTexCoords0[tri_vert];
			varSpotLightShadowMapTexCoords1 = geoSpotLightShadowMapTexCoords1[tri_vert];
			varPositionVS = geoPositionVS[tri_vert];
			varNormalVS = geoNormalVS[tri_vert];

			WS_pos_from_GS = gl_in[tri_vert].gl_Position;
			gl_Position = cmMat[gl_Layer] * WS_pos_from_GS;
			//gl_Position = cmMat * WS_pos_from_GS;
			EmitVertex();
		}
		EndPrimitive();
	}
}