#version 400 core

uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat4 projMatrix;
//uniform mat4 dirLightTextureModelMatrix;  // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)
uniform mat4 spotLightTextureModelMatrix0; // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)
uniform mat4 spotLightTextureModelMatrix1; // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)

uniform vec3 cameraPos;
uniform vec3 cameraUp;
uniform vec3 cameraFront;

layout(points) in;
layout(triangle_strip, max_vertices=4) out;

in float varSize[];
in float varMass[];

out vec3 varPositionWS;
out vec2 varTexCoords;
//out vec4 varDirLightShadowMapTexCoords;
out vec4 varSpotLightShadowMapTexCoords0;
out vec4 varSpotLightShadowMapTexCoords1;

void main(void) 
{
	vec3 eyeToVertex = vec3(gl_in[0].gl_Position) - cameraPos;
	// a very simple form of view frustum culling - using only the cosine to the cameraFront vector
	if (dot(normalize(eyeToVertex), cameraFront) > 0.62) // If you change the viewport, you'll have to change the cosine-threshold
	{
		// do spherical billboarding
		vec3 sideVector = normalize(cross(eyeToVertex, cameraUp));
		vec3 upVector = normalize(cross(sideVector, eyeToVertex));

		mat4 M = projMatrix * viewMatrix;
		vec4 pos;

		pos = (gl_in[0].gl_Position + vec4( - sideVector * varSize[0] - upVector * varSize[0], 0)); // vec4(-varSize[0], -varSize[0], 0.0, 0.0));
		gl_Position = M * pos;
		varPositionWS = pos.xyz;
		varTexCoords = vec2(0.0, 0.0);
		//varDirLightShadowMapTexCoords = dirLightTextureModelMatrix * pos;
		varSpotLightShadowMapTexCoords0 = spotLightTextureModelMatrix0 * pos;
		varSpotLightShadowMapTexCoords1 = spotLightTextureModelMatrix1 * pos;
		EmitVertex();

		pos = (gl_in[0].gl_Position + vec4( + sideVector * varSize[0] - upVector * varSize[0], 0)); // + vec4( varSize[0], -varSize[0], 0.0, 0.0));
		gl_Position = M * pos;
		varPositionWS = pos.xyz;
		varTexCoords = vec2(1.0, 0.0);
		//varDirLightShadowMapTexCoords = dirLightTextureModelMatrix * pos;
		varSpotLightShadowMapTexCoords0 = spotLightTextureModelMatrix0 * pos;
		varSpotLightShadowMapTexCoords1 = spotLightTextureModelMatrix1 * pos;
		EmitVertex();

		pos = (gl_in[0].gl_Position + vec4( - sideVector * varSize[0] + upVector * varSize[0], 0)); // + vec4(-varSize[0],  varSize[0], 0.0, 0.0));
		gl_Position = M * pos;
		varPositionWS = pos.xyz;
		varTexCoords = vec2(0.0, 1.0);
		//varDirLightShadowMapTexCoords = dirLightTextureModelMatrix * pos;
		varSpotLightShadowMapTexCoords0 = spotLightTextureModelMatrix0 * pos;
		varSpotLightShadowMapTexCoords1 = spotLightTextureModelMatrix1 * pos;
		EmitVertex();

		pos = (gl_in[0].gl_Position + vec4( + sideVector * varSize[0] + upVector * varSize[0], 0)); // + vec4( varSize[0],  varSize[0], 0.0, 0.0));
		gl_Position = M * pos;
		varPositionWS = pos.xyz;
		varTexCoords = vec2(1.0, 1.0);
		//varDirLightShadowMapTexCoords = dirLightTextureModelMatrix * pos;
		varSpotLightShadowMapTexCoords0 = spotLightTextureModelMatrix0 * pos;
		varSpotLightShadowMapTexCoords1 = spotLightTextureModelMatrix1 * pos;
		EmitVertex();
	}
	
	EndPrimitive();
}