#version 400

uniform vec4 ambientEmissiveLightColor;
uniform vec4 matDiffuseColor;
uniform vec4 matSpecularColor;
uniform float shininess;
uniform float pixelSize;
uniform int softness;

uniform vec3 dirLight0Direction; ///< light-direction in eye-space, must be normalized
uniform vec4 dirLight0DiffuseColor;
uniform vec4 dirLight0SpecularColor;

uniform sampler2D texSampler0;
uniform sampler2D shadowSampler0;

in vec2 varTexCoords;
in vec4 shadowMapTexCoords;
in vec4 varPositionVS; ///< vertex position in viewing-space
in vec3 varNormalVS;   ///< vertex normal in viewing-space

float inShadowglobal = 1.0;

int BLOCKER_SEARCH_NUM_SAMPLES = 16;
int PCF_NUM_SAMPLES = 16;
float NEAR_PLANE = 3.0;
float LIGHT_WORLD_SIZE = 0.5;
float LIGHT_FRUSTUM_WIDTH = 3.75;
float LIGHT_SIZE_UV = LIGHT_WORLD_SIZE / LIGHT_FRUSTUM_WIDTH; 


float avgBlockerDepth = 0.0;
float numBlockers = 0.0;

//Parallel plane estimation
float PenumbraSize(float zReceiver, float zBlocker) 
{ 
	return (zReceiver - zBlocker) * LIGHT_WORLD_SIZE / zBlocker;
} 

// We search the shadow map and
// average the depths that are closer to the light source than to
// the point being shaded (the receiver).
void FindBlocker( vec2 uv, float zReceiver ) 
{ 


	/// TODO: const array
	vec2 poissonDisk[] = vec2[]
	(
	 vec2( -0.94201624, -0.39906216 ), 
	 vec2( 0.94558609, -0.76890725 ), 
	 vec2( -0.094184101, -0.92938870 ),
	 vec2( 0.34495938, 0.29387760 ),
	 vec2( -0.91588581, 0.45771432 ), 
	 vec2( -0.81544232, -0.87912464 ), 
	 vec2( -0.38277543, 0.27676845 ),
	 vec2( 0.97484398, 0.75648379 ),
	 vec2( 0.44323325, -0.97511554 ), 
	 vec2( 0.53742981, -0.47373420 ), 
	 vec2( -0.26496911, -0.41893023 ), 
	 vec2( 0.79197514, 0.19090188 ), 
	 vec2( -0.24188840, 0.99706507 ), 
	 vec2( -0.81409955, 0.91437590 ), 
	 vec2( 0.19984126, 0.78641367 ),
	 vec2( 0.14383161, -0.14100790 )
	);

    // This uses similar triangles to compute what  
    // area of the shadow map we should search 
	// The size of the
	// search region depends on the light size and the 
	// receivers distance from the light source.
	
	float searchWidth = (zReceiver - 1.0) * LIGHT_SIZE_UV / zReceiver; // gute .... why ?
	//float searchWidth = (zReceiver - NEAR_PLANE) * LIGHT_SIZE_UV / zReceiver; // schlechte results
	float blockerSum = 0; 
    numBlockers = 0.0; 
     
	// average the depths that are closer to the light source than to the receiver
    for( int i = 0; i < BLOCKER_SEARCH_NUM_SAMPLES; ++i ) 
    { 

		float shadowMapDepth = texture2D(shadowSampler0, uv + poissonDisk[i] * searchWidth*2.0).r;
        if ( shadowMapDepth < zReceiver) { 
                blockerSum += shadowMapDepth; 
                numBlockers++; 
            } 
     } 
    avgBlockerDepth = blockerSum / numBlockers; 
}

// PCF Filtering
float PCF_Filter( vec2 uv, float zReceiver, float filterRadiusUV ) 
{ 
	/// TODO: const array
	vec2 poissonDisk[] = vec2[]
	(
	 vec2( -0.94201624, -0.39906216 ), 
	 vec2( 0.94558609, -0.76890725 ), 
	 vec2( -0.094184101, -0.92938870 ),
	 vec2( 0.34495938, 0.29387760 ),
	 vec2( -0.91588581, 0.45771432 ), 
	 vec2( -0.81544232, -0.87912464 ), 
	 vec2( -0.38277543, 0.27676845 ),
	 vec2( 0.97484398, 0.75648379 ),
	 vec2( 0.44323325, -0.97511554 ), 
	 vec2( 0.53742981, -0.47373420 ), 
	 vec2( -0.26496911, -0.41893023 ), 
	 vec2( 0.79197514, 0.19090188 ), 
	 vec2( -0.24188840, 0.99706507 ), 
	 vec2( -0.81409955, 0.91437590 ), 
	 vec2( 0.19984126, 0.78641367 ),
	 vec2( 0.14383161, -0.14100790 )
	);


    float sum = 0.0f; 
	for ( int i = 0; i < PCF_NUM_SAMPLES; ++i ) 
    { 
		
        vec2 offset = poissonDisk[i] * filterRadiusUV; 

		// Bad Filtering
        //sum += (texture2D(shadowSampler0, uv + offset).r < zReceiver) ? 0.0 : 1.0;
		
		// Better Filtering
		float depthglobal = texture2D(shadowSampler0, uv + offset).r;
		float lal = (depthglobal < zReceiver) ? 0.0 : 1.0;
		depthglobal = texture2D(shadowSampler0, uv + offset + vec2(pixelSize,pixelSize)).r;
		lal += (depthglobal < zReceiver) ? 0.0 : 1.0;
		depthglobal = texture2D(shadowSampler0, uv + offset + vec2(pixelSize,-pixelSize)).r;
		lal += (depthglobal < zReceiver) ? 0.0 : 1.0;
		depthglobal = texture2D(shadowSampler0, uv + offset + vec2(-pixelSize,pixelSize)).r;
		lal += (depthglobal < zReceiver) ? 0.0 : 1.0;
		depthglobal = texture2D(shadowSampler0, uv + offset + vec2(-pixelSize,-pixelSize)).r;
		lal += (depthglobal < zReceiver) ? 0.0 : 1.0;
		/*depthglobal = texture2D(shadowSampler0, uv + offset + vec2(pixelSize,0)).r;
		lal += (depthglobal < zReceiver) ? 0.0 : 1.0;
		depthglobal = texture2D(shadowSampler0, uv + offset + vec2(0,pixelSize)).r;
		lal += (depthglobal < zReceiver) ? 0.0 : 1.0;
		depthglobal = texture2D(shadowSampler0, uv + offset + vec2(-pixelSize,0)).r;
		lal += (depthglobal < zReceiver) ? 0.0 : 1.0;
		depthglobal = texture2D(shadowSampler0, uv + offset + vec2(0,-pixelSize)).r;
		lal += (depthglobal < zReceiver) ? 0.0 : 1.0;*/

		lal = lal / 5.0;
		sum += lal;
    } 

//    return sum / PCF_NUM_SAMPLES; 
return sum / PCF_NUM_SAMPLES; 
} 

float PCSS () 
{ 
	vec2 uv = shadowMapTexCoords.xy / shadowMapTexCoords.w; 
	float zReceiver = shadowMapTexCoords.z / shadowMapTexCoords.w; // Assumed to be eye-space z in this code 

	avgBlockerDepth = 0.0;
	numBlockers = 0.0;


	// STEP 1: blocker search 
	FindBlocker(uv, zReceiver ); 
	if( numBlockers < 1 )   
		return 1.0f; //There are no occluders so early out (this saves filtering) 
	

	// STEP 2: penumbra size 
	float penumbraRatio = PenumbraSize(zReceiver, avgBlockerDepth);     
	float filterRadiusUV = penumbraRatio * LIGHT_SIZE_UV * NEAR_PLANE / zReceiver; 
	//float filterRadiusUV = PenumbraSize(zReceiver, avgBlockerDepth);     
     
	// STEP 3: filtering 
	return PCF_Filter( uv, zReceiver, filterRadiusUV ); 
}

void main()
{
	inShadowglobal = 1.0;
	inShadowglobal = PCSS();

	// calculate lighting
	vec3 eyePosVS = vec3(0.0, 0.0, 0.0);
	vec3 toEyeDir = normalize(eyePosVS - varPositionVS.xyz);
	vec3 toLightDir = -dirLight0Direction;
	vec3 normal = normalize(varNormalVS);
	vec3 halfVector = normalize(toLightDir + toEyeDir);
	
	float NdotL = max(0.0, dot(toLightDir, normal)); // lambertian coefficient
	float NdotH = max(0.0, dot(halfVector, normal));
	float specularCoeff = pow(NdotH, shininess); // specular coefficient
	
	// calculate diffuse and ambient color contribution with diffuse coefficients of light and material
	vec4 diffuseColor = dirLight0DiffuseColor * matDiffuseColor * NdotL;
	vec4 ambientColor = ambientEmissiveLightColor;
	diffuseColor = diffuseColor * texture2D(texSampler0, varTexCoords) * inShadowglobal;
	ambientColor = ambientColor * texture2D(texSampler0, varTexCoords) * inShadowglobal;
	
	// calculate specular color contribution with specular coefficients of light and material
	vec4 specularColor = dirLight0SpecularColor * matSpecularColor * specularCoeff * inShadowglobal;

	// add all together
	//gl_FragColor = ambientColor + diffuseColor + specularColor;
	gl_FragColor = vec4(1.0,1.0,1.0,1.0) * inShadowglobal;
}