#version 400

uniform vec4 ambientEmissiveLightColor;
uniform vec4 matDiffuseColor;
uniform vec4 matSpecularColor;
uniform vec4 lightPosition;
uniform float shininess;
uniform float pixelSize;
uniform int softness;

uniform vec3 dirLight0Direction; ///< light-direction in eye-space, must be normalized
uniform vec4 dirLight0DiffuseColor;
uniform vec4 dirLight0SpecularColor;

uniform int useTexture;
uniform sampler2D texSampler0;
uniform samplerCube shadowSampler0;

in vec2 varTexCoords;
in vec4 shadowMapTexCoords;
in vec4 varWorldPos;
in vec4 varPositionVS; ///< vertex position in viewing-space
in vec3 varNormalVS;   ///< vertex normal in viewing-space

//TODO: UINIFORM FOR CUBEMAP NEAR FAR
float near = 0.3;
float far = 100.0;

float inShadowglobal = 1.0;

int BLOCKER_SEARCH_NUM_SAMPLES = 16;
int PCF_NUM_SAMPLES = 16;
float NEAR_PLANE = 3.0;
float LIGHT_WORLD_SIZE = 0.5;
float LIGHT_FRUSTUM_WIDTH = 3.75;
float LIGHT_SIZE_UV = LIGHT_WORLD_SIZE / LIGHT_FRUSTUM_WIDTH; 

float avgBlockerDepth = 0.0;
float numBlockers = 0.0;

vec3 cm_lookup_vec;

// ------------- fragment shader output -----------------
out vec4 fragColor;

//Parallel plane estimation
float PenumbraSize(float zReceiver, float zBlocker) 
{ 
	return (zReceiver - zBlocker) * LIGHT_WORLD_SIZE / zBlocker;
} 

// We search the shadow map and
// average the depths that are closer to the light source than to
// the point being shaded (the receiver).
void FindBlocker( vec3 uv, float zReceiver ) 
{ 

	// Random Vectors Within Unit Sphere (stolen from GameRendering)
	vec3 randVec[16] = vec3[](vec3(0.53812504, 0.18565957, -0.43192),vec3(0.13790712, 0.24864247, 0.44301823),vec3(0.33715037, 0.56794053, -0.005789503),vec3(-0.6999805, -0.04511441, -0.0019965635),vec3(0.06896307, -0.15983082, -0.85477847),vec3(0.056099437, 0.006954967, -0.1843352),vec3(-0.014653638, 0.14027752, 0.0762037),vec3(0.010019933, -0.1924225, -0.034443386),vec3(-0.35775623, -0.5301969, -0.43581226),vec3(-0.3169221, 0.106360726, 0.015860917),vec3(0.010350345, -0.58698344, 0.0046293875),vec3(-0.08972908, -0.49408212, 0.3287904),vec3(0.7119986, -0.0154690035, -0.09183723),vec3(-0.053382345, 0.059675813, -0.5411899),vec3(0.035267662, -0.063188605, 0.54602677),vec3(-0.47761092, 0.2847911, -0.0271716));

    // This uses similar triangles to compute what  
    // area of the shadow map we should search 
	// The size of the
	// search region depends on the light size and the 
	// receivers distance from the light source.
	
	float searchWidth = (zReceiver - 1.0) * LIGHT_SIZE_UV / zReceiver; // gute .... why ?
	searchWidth = searchWidth * 0.1;
	//float searchWidth = (zReceiver - NEAR_PLANE) * LIGHT_SIZE_UV / zReceiver; // schlechte results
	float blockerSum = 0; 
    numBlockers = 0.0; 
     
	// average the depths that are closer to the light source than to the receiver
    for( int i = 0; i < BLOCKER_SEARCH_NUM_SAMPLES; ++i ) 
    { 
		float eps = 0.15;
		float shadowMapDepth = texture(shadowSampler0, uv + randVec[i]* searchWidth).r * far + eps;
		//float shadowMapDepth = texture(shadowSampler0, uv + randVec[i]* searchWidth*2.0).r * far;
        if ( shadowMapDepth < zReceiver) { 
                blockerSum += shadowMapDepth; 
                numBlockers++; 
            } 
     } 
    avgBlockerDepth = blockerSum / numBlockers; 
}

// PCF Filtering
float PCF_Filter( vec3 uv, float zReceiver, float filterRadiusUV ) 
{ 
	/// TODO: const array
	// Random Vectors Within Unit Sphere (stolen from GameRendering)
	vec3 randVec[16] = vec3[](vec3(0.53812504, 0.18565957, -0.43192),vec3(0.13790712, 0.24864247, 0.44301823),vec3(0.33715037, 0.56794053, -0.005789503),vec3(-0.6999805, -0.04511441, -0.0019965635),vec3(0.06896307, -0.15983082, -0.85477847),vec3(0.056099437, 0.006954967, -0.1843352),vec3(-0.014653638, 0.14027752, 0.0762037),vec3(0.010019933, -0.1924225, -0.034443386),vec3(-0.35775623, -0.5301969, -0.43581226),vec3(-0.3169221, 0.106360726, 0.015860917),vec3(0.010350345, -0.58698344, 0.0046293875),vec3(-0.08972908, -0.49408212, 0.3287904),vec3(0.7119986, -0.0154690035, -0.09183723),vec3(-0.053382345, 0.059675813, -0.5411899),vec3(0.035267662, -0.063188605, 0.54602677),vec3(-0.47761092, 0.2847911, -0.0271716));


	float eps = 0.15;
    float sum = 0.0f; 
	for ( int i = 0; i < PCF_NUM_SAMPLES; ++i ) 
    { 
		
        vec3 offset = randVec[i];//* filterRadiusUV; 

		// Bad Filtering
       // sum += (texture(shadowSampler0, uv + offset).r * far + eps < zReceiver) ? 0.0 : 1.0;
			
		
		// Better Filtering
		float depthglobal = texture(shadowSampler0, uv + offset).r;
		float lal = (depthglobal * far + eps < zReceiver) ? 0.0 : 1.0;
		depthglobal = texture(shadowSampler0, uv + offset + vec3(pixelSize,pixelSize, pixelSize)).r;
		lal += (depthglobal * far + eps < zReceiver) ? 0.0 : 1.0;
		depthglobal = texture(shadowSampler0, uv + offset + vec3(pixelSize,-pixelSize, pixelSize)).r;
		lal += (depthglobal * far + eps < zReceiver) ? 0.0 : 1.0;
		depthglobal = texture(shadowSampler0, uv + offset + vec3(-pixelSize,pixelSize, -pixelSize)).r;
		lal += (depthglobal * far + eps < zReceiver) ? 0.0 : 1.0;
		depthglobal = texture(shadowSampler0, uv + offset + vec3(-pixelSize,-pixelSize, -pixelSize)).r;
		lal += (depthglobal * far + eps < zReceiver) ? 0.0 : 1.0;
		/*depthglobal = texture(shadowSampler0, uv + offset + vec3(pixelSize,0 , pixelSize)*2.0).r;
		lal += (depthglobal * far < zReceiver) ? 0.0 : 1.0;
		depthglobal = texture(shadowSampler0, uv + offset + vec3(0,pixelSize, - pixelSize)*2.0).r;
		lal += (depthglobal * far < zReceiver) ? 0.0 : 1.0;
		depthglobal = texture(shadowSampler0, uv + offset + vec3(-pixelSize,0 , 0)*2.0).r;
		lal += (depthglobal * far < zReceiver) ? 0.0 : 1.0;
		depthglobal = texture(shadowSampler0, uv + offset + vec3(0,-pixelSize, pixelSize)*2.0).r;
		lal += (depthglobal * far < zReceiver) ? 0.0 : 1.0;*/

		lal = lal / 5.0;
		sum += lal;
    } 
	return sum / PCF_NUM_SAMPLES; 
} 

float PCSS () 
{ 



	vec3 uv = varWorldPos.xyz - lightPosition.xyz;
	

	float zReceiver = length(uv); // Assumed to be eye-space z in this code 

	avgBlockerDepth = 0.0;
	numBlockers = 0.0;


	// STEP 1: blocker search 
	FindBlocker(normalize(uv), zReceiver ); 
	
	if( numBlockers < 1 )   
		return 1.0; //There are no occluders so early out (this saves filtering) 
	

	// STEP 2: penumbra size 
	float penumbraRatio = PenumbraSize(zReceiver, avgBlockerDepth);     
	float filterRadiusUV = penumbraRatio * LIGHT_SIZE_UV * NEAR_PLANE / zReceiver; 
	//float filterRadiusUV = PenumbraSize(zReceiver, avgBlockerDepth);     
     
	// STEP 3: filtering 
	return PCF_Filter( normalize(uv), zReceiver, filterRadiusUV ); 
}


void main()
{
	//inShadowglobal = PCSS(); //Not working for Cubemaps yet.... use NON omnidirectional Shadows for Softshadows
	
	cm_lookup_vec = varWorldPos.xyz - lightPosition.xyz;
	float smallest_dist_to_light = texture(shadowSampler0, cm_lookup_vec).r;
	float curr_fragment_dist_to_light = length(cm_lookup_vec);
	
	// undo previous [0;1]-mapping of 'dist-to-lightsource'
	smallest_dist_to_light = smallest_dist_to_light * far;

	float eps = 0.15; // add a small offset (adjust as needed)
	
	vec3 position = varWorldPos.xyz / varWorldPos.w;
	vec3 light_dir = normalize(lightPosition.xyz - position);
	vec4 diffuse = vec4(0.4,0.4,0.0,1.0) * max(0.0, dot(light_dir,varNormalVS));


	if(smallest_dist_to_light + eps < curr_fragment_dist_to_light)
		inShadowglobal = 0.0;
	else
		inShadowglobal = 1.0;

	fragColor = diffuse * inShadowglobal;

}