#version 400 core

uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat4 projMatrix;
//uniform mat4 textureModelMatrix; // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)
uniform mat4 dirLightTextureModelMatrix;  // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)
uniform mat4 spotLightTextureModelMatrix0; // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)
uniform mat4 spotLightTextureModelMatrix1; // texture matrix * model matrix (tex mat : m_textureMatrix * m_lightPerspective * m_lightView;)

uniform vec3 cameraPos;
uniform vec3 cameraUp;
//uniform vec3 cameraFront;

layout(points) in;
layout(triangle_strip, max_vertices=4) out;

in float varSize[];
in float varAlpha[];
in float varRotation[];

out vec3 varPositionWS;
out vec2 varTexCoords;
//out vec4 varShadowMapTexCoords;
out vec4 varDirLightShadowMapTexCoords;
out vec4 varSpotLightShadowMapTexCoords0;
out vec4 varSpotLightShadowMapTexCoords1;
out float alpha;

void main(void) 
{
	// do spherical billboarding
	vec3 eyeToVertex = vec3(gl_in[0].gl_Position) - cameraPos;	
	vec3 sideVector = normalize(cross(eyeToVertex, cameraUp));
	vec3 upVector = normalize(cross(sideVector, eyeToVertex));

	// these are for rotating the billboard
	float angle = varRotation[0];
	float cosA = cos(angle);
	float sinA = sin(angle);
	vec3 rotSideVec = cosA * sideVector - sinA * upVector;
	vec3 rotUpVec = sinA * sideVector + cosA * upVector;

	mat4 M = projMatrix * viewMatrix;
	vec4 pos;

	pos = (gl_in[0].gl_Position + vec4( - rotSideVec * varSize[0] - rotUpVec * varSize[0], 0)); // vec4(-varSize[0], -varSize[0], 0.0, 0.0));
	varPositionWS = pos.xyz;
	gl_Position = M * pos;
	varTexCoords = vec2(0.0, 0.0);
	//varShadowMapTexCoords = textureModelMatrix * pos;
	varDirLightShadowMapTexCoords = dirLightTextureModelMatrix * pos;
	varSpotLightShadowMapTexCoords0 = spotLightTextureModelMatrix0 * pos;
	varSpotLightShadowMapTexCoords1 = spotLightTextureModelMatrix1 * pos;
	alpha = varAlpha[0];
	EmitVertex();

	pos = (gl_in[0].gl_Position + vec4( + rotSideVec * varSize[0] - rotUpVec * varSize[0], 0)); // + vec4( varSize[0], -varSize[0], 0.0, 0.0));
	varPositionWS = pos.xyz;
	gl_Position = M * pos;
	varTexCoords = vec2(1.0, 0.0);
	//varShadowMapTexCoords = textureModelMatrix * pos;
	varDirLightShadowMapTexCoords = dirLightTextureModelMatrix * pos;
	varSpotLightShadowMapTexCoords0 = spotLightTextureModelMatrix0 * pos;
	varSpotLightShadowMapTexCoords1 = spotLightTextureModelMatrix1 * pos;
	alpha = varAlpha[0];
	EmitVertex();

	pos = (gl_in[0].gl_Position + vec4( - rotSideVec * varSize[0] + rotUpVec * varSize[0], 0)); // + vec4(-varSize[0],  varSize[0], 0.0, 0.0));
	varPositionWS = pos.xyz;
	gl_Position = M * pos;
	varTexCoords = vec2(0.0, 1.0);
	//varShadowMapTexCoords = textureModelMatrix * pos;
	varDirLightShadowMapTexCoords = dirLightTextureModelMatrix * pos;
	varSpotLightShadowMapTexCoords0 = spotLightTextureModelMatrix0 * pos;
	varSpotLightShadowMapTexCoords1 = spotLightTextureModelMatrix1 * pos;
	alpha = varAlpha[0];
	EmitVertex();

	pos = (gl_in[0].gl_Position + vec4( + rotSideVec * varSize[0] + rotUpVec * varSize[0], 0)); // + vec4( varSize[0],  varSize[0], 0.0, 0.0));
	varPositionWS = pos.xyz;
	gl_Position = M * pos;
	varTexCoords = vec2(1.0, 1.0);
	//varShadowMapTexCoords = textureModelMatrix * pos;
	varDirLightShadowMapTexCoords = dirLightTextureModelMatrix * pos;
	varSpotLightShadowMapTexCoords0 = spotLightTextureModelMatrix0 * pos;
	varSpotLightShadowMapTexCoords1 = spotLightTextureModelMatrix1 * pos;
	alpha = varAlpha[0];
	EmitVertex();
	
	EndPrimitive();
}