#version 400

//uniform float ssaoFocus;
//uniform float ssaoPower;

//uniform vec2 pixelSize;

uniform mat4 inverseViewMatrix;
uniform mat4 inverseProjMatrix;
uniform mat4 projectionMatrix;

uniform float strength;
uniform float area;
uniform float falloff;
uniform float radius;

uniform sampler2D texSampler0; // RGB Surface Normals in View Space Important!! Alpha = Linear Depth
uniform sampler2D texSampler1; // random Normal Texture
//uniform sampler2D texSampler2; // DepthBuffer

varying vec2 varTexCoords;




// ------------- output-color of fragment ------------
out vec4 fragColor;


void main()
{
	const int samples = 32; 


	float depth = texture2D(texSampler0, varTexCoords).a;
 	vec3 position = vec3(varTexCoords, depth);
 	vec3 normal = normalize(texture2D(texSampler0, varTexCoords).xyz) * 2.0 - 1.0;
  
	// random uv cords for ssao random vector...
	vec2 modifier = texture2D(texSampler1, varTexCoords / 0.00003).xy + (position.xy + position.z);

	float radiusDepth = radius/depth;
	float occlusion = 0.0;
	vec4 random = vec4(1.0); 
	
	for(int i=0; i < samples; i++) 
	{
		// fetch a random vector
		random = texture2D(texSampler1, modifier);
 
		// Normalize vs non Normalize ... which one looks better ?
		random.xyz = normalize(random.xyz) * 2.0 - 1.0;
 
		// more randomness for next loop
		modifier += random.xy;

		// Flip the random vector if it's below the plane
		if (dot(random.xyz, normal) < 0.0) 
			random.xyz = -random.xyz;

		// Ray should now be in the same hemisphere as normal
		vec3 hemiRay = random.xyz * (radiusDepth * random.w) + position;

		// get depth and normal of new possible occluder
		float occDepth = texture2D(texSampler0, hemiRay.xy).a;
		vec3 occNormal = normalize(texture2D(texSampler0, hemiRay.xy).xyz) * 2.0 - 1.0;

		float dDifference = depth - occDepth;

		// User normal difference as weight for ao ... test with and without.. whichone looks better ?
		float nWeight = 1.0 - dot(occNormal,normal);
    
		occlusion += step(falloff, dDifference) * (1.0-smoothstep(falloff, area, dDifference)) * nWeight;
	}
	
	float ao = 1.0 - strength * occlusion * (1.0 / samples);
	fragColor = vec4(ao,ao,ao,1.0);
	//fragColor = vec4(0.0, 0.0, 0.0, 1.0 - ao);
}
